import sys
import pygame
import random
from math import sin, cos, atan2
from cmath import exp as cexp
from math import pi
from types import *
import data
df=data.filepath

debug=False
SW=640
SH=480
#BASEROT=pi/180
#BASEROT=pi/9
qoffset=-3-80j
SAFEZONE=20
PTHRESH=1e-6
PARTHRESH=1e-7
BASESPEED=2
BASEROT=pi/90*BASESPEED
cc=complex.conjugate
TB=25
textbox=pygame.Rect(TB,SH-150,SW-2*TB,150-TB)
boardsize=(SW*2,SH*2)
INVULTURNS=100

arcangle=pi/6
CIRCSPIN=pi/90

allschemes=["testlevel",
"a+b+a-b-",
"a-b-a+b+",
"a+b-a+b+",
"a+b+c+a-b-c-",
"a+b+c+a+b+c-",
"a+a-b+c+c-b+",
"a+b+c+d+a-b-c-d-",
"timeattack"]
JUMPLEVEL=6
JUMPDELAY=500

LEVELUPPAUSE=100
LEVELUPDELAY=50

GAMEOVERPAUSE=200

def ccw(A,B,C):
    return (C.imag-A.imag)*(B.real-A.real) > (B.imag-A.imag)*(C.real-A.real)

#Is A between B and C
def between(A,B,C):
    return (B.real<A.real<C.real and B.imag<A.imag<C.imag) or (C.real<A.real<B.real and C.imag<A.imag<B.imag)

def c2t(c):
    return (c.real,c.imag)

def anglebetween(a,b):
    #atan2(perpdot(a,b),dot(a,b))
    if debug: print "ab",a,b,a*1j
    if debug: print (dot(a,b)+1j*dot((a*1j),b))/abs(a)/abs(b)
    return (dot(a,b)+1j*dot((a*1j),b))/abs(a)/abs(b)
#return (dot((a*1j),b),dot(a,b))

def dot(a,b):
    return a.real*b.real+a.imag*b.imag

def polar(r,t):
    return r*cexp(2*pi*t*1j)
def polypolar(r,i,n):
    return r*cexp(2*pi*i*1j/n)

def rendertext(surf,texts,font,box,col=(0,0,0)):
    l=0
    h=font.size(" ")[1]
    for s in texts.split("\n"):
        text=font.render(s,1,col)
        surf.blit(text,(box.left+4,box.top+4+h*l))
        l+=1

def schemetopoly(s):
    polyn=len(s)/2
    d={}
    bjump=[-1]*polyn
    bflip=[True]*polyn
    arrows=[-1]*polyn
    arrown=1
    for i in xrange(polyn):
        ch,sign=s[2*i],s[2*i+1]
        if ch in d:
            bjump[d[ch][0]]=i
            bjump[i]=d[ch][0]
            bflip[d[ch][0]]=bflip[i]=(sign==d[ch][1])
            arrows[i]=arrows[d[ch][0]]=arrown
            arrown+=1
        else:
            d[ch] = (i,sign)
    if debug: print "bjump",bjump
    if debug: print "bflip",bflip
    if debug: print "arrows",arrows
    return (polyn,bjump,bflip,arrows)

class Segm:
    def __init__(self,init,final,diffmode=True,arrow=0):
        self.init=init
        if diffmode:
            self.diff=final
            self.final=init+final
        else:
            self.final=final
            self.diff=final-init
        self.arrow=arrow

    def __repr__(self):
        return str((self.init,self.final))

    def __contains__(self,others):
        #"in" now means "intersect" rather than "contained in"
        #if others.type==ListType:
        #    return any(self in x for x in others)
        #else:
        
        #Approximate parallel
        if 1-dot(self.diff,others.diff)/abs(self.diff)/abs(others.diff)<PARTHRESH:
            #dot(others.init-self.init,others.diff)/abs(others.diff)<PARTHRESH 
            return ccw(self.init,others.init,others.final) != ccw(self.final,others.init,others.final) and ccw(self.init,self.final,others.init) != ccw(self.init,self.final,others.final) and (between(others.init,self.init,self.final) or between(others.final,self.init,self.final))
        #Learned from http://www.bryceboe.com/2006/10/23/line-segment-intersection-algorithm/
        else:
            return ccw(self.init,others.init,others.final) != ccw(self.final,others.init,others.final) and ccw(self.init,self.final,others.init) != ccw(self.init,self.final,others.final)

    def intpt(self,other):
        #Assuming "other" is more reliable than self.
        #Might need something more stable numerically.
        #init1+t1*diff1=init2+t2*diff2
        if abs(other.diff.real)<=PTHRESH:
            if debug: print "case 1",self.diff,other.diff
            t1=(other.init.real-self.init.real)/self.diff.real
            t2=(self.init.imag+t1*self.diff.imag-other.init.imag)/other.diff.imag
        elif abs(other.diff.imag)<=PTHRESH:
            if debug: print "case 2",self.diff,other.diff
            t1=(other.init.imag-self.init.imag)/self.diff.imag
            t2=(self.init.real+t1*self.diff.real-other.init.real)/other.diff.real
        else:
            #b=other.init-self.init
            #(b[0]+t1*diff1[0])/diff2[0]=(b[1]+t1*diff1[1])/diff2[1]
            #t1*(diff1[0]/diff2[0]-diff1[1]/diff2[1])=(b[1]/diff2[1]-b[0]/diff2[0])
            if debug: print "case 3",self.diff,other.diff
            if debug: print self,other
            b=self.init-other.init
            t1=((b.imag/other.diff.imag)-(b.real/other.diff.real))/(self.diff.real/other.diff.real-self.diff.imag/other.diff.imag)
            t2=(self.init.imag+t1*self.diff.imag-other.init.imag)/other.diff.imag
        return (t1,t2,self.init+t1*self.diff)

    def draw(self,dsurf,col=(0,0,0),printing=False):
        printing=False
        init=self.init+g.origin
        final=self.final+g.origin
        if printing and debug: print init,final
        pygame.draw.line(dsurf,col,c2t(init),c2t(final))
        for i in xrange(self.arrow):
            if i%2==0:
                midpt=init+(0.5+0.05*i/2)*self.diff
            else:
                midpt=init+(0.5-0.05*(i+1)/2)*self.diff
            if printing and debug: print "arrow",midpt,midpt+100*self.diff/abs(self.diff+50*self.diff*1j/abs(self.diff))
            if printing and debug: print init,self.diff,final
            pygame.draw.line(dsurf,col,c2t(midpt),c2t(midpt+10*(2*self.diff/abs(self.diff)+self.diff*1j/abs(self.diff))))
            pygame.draw.line(dsurf,col,c2t(midpt),c2t(midpt+10*(2*self.diff/abs(self.diff)-self.diff*1j/abs(self.diff))))

class Game:
    origin=SW+SH*1j
    #bjump=[1,0,3,2,5,4]

    def __init__(self):
        self.timeattackmode=False
        self.totalturn=0
        self.turn=0
        self.level=1
        self.state=0
        self.waiting=False
        self.paused=0
        self.sounds={}
        self.loadsounds()
        self.loadscript()
        self.maxlevel=1

        self.soundon=True
        self.bgmon=True

        """
        self.zsurf=pygame.Surface(boardsize,screen.get_flags(),screen)
        self.zsurf.set_colorkey((250,250,250))
        self.parity=True
        self.wcount=0
        self.zones=[]
        self.bjump=[]
        self.bflip=[]
        """
        #self.loadlevel(self.level)

        self.scorefont = pygame.font.Font(None, 25)
        self.delayfont = pygame.font.Font(None, 25)
        self.quillfont = pygame.font.Font(df("cacpinaf.ttf"), 23)
        self.dialogfont = pygame.font.Font(df("copacetix.ttf"), 20)
        self.menufont = pygame.font.Font(df("cacpinaf.ttf"), 40)
        #self.titlefont = pygame.font.Font("wirefram.ttf", 100,bold=True)
        self.titlefont = pygame.font.Font(df("level.ttf"),90,bold=True)
        self.pausefont = pygame.font.Font(None, 35)
        self.quotefont = pygame.font.Font(df("ly.ttf"),20)
        
        self.tsurf=pygame.Surface((SW,SH),screen.get_flags(),screen)
        self.tsurf.set_colorkey((250,250,250))
        self.dialogvisible=False
        #self.drawtextbox("Hello")
        #self.mq=None

        self.psurf=pygame.Surface((SW,SH),screen.get_flags(),screen)
        self.psurf.set_colorkey((250,250,250))
        self.loadpause()

        self.fadeonce=True
        self.title=self.titlefont.render("<Topologiquill>",1,(0,0,0))

        #Probably don't need this big
        self.hudsurf=pygame.Surface((SW,SH),screen.get_flags(),screen)
        self.hudsurf.set_colorkey((250,250,250))

        self.special=[]
        self.asurf=pygame.Surface((SW,SH),screen.get_flags(),screen)
        self.asurf.fill([255,255,255])
        self.asurf.set_colorkey((250,250,250))

        self.loadlevelup()
        self.loadgameover()

        self.levelingup=False
        self.coversurf=pygame.Surface((SW,SH),screen.get_flags(),screen)
        self.coversurf.fill([255,255,255])

    def loadlevelup(self):
        self.levelupsurf=pygame.Surface((SW,SH/4),screen.get_flags(),screen)
        self.levelupfont=pygame.font.Font(df("copacetix.ttf"),60)
        text="LEVEL SURFACE UP"
        col=(255,255,255)
        self.levelupsurf.blit(self.levelupfont.render(text,1,col),(20,30))

        """
        self.winsurf=pygame.Surface((SW,SH/4),screen.get_flags(),screen)
        font=pygame.font.Font("copacetix.ttf",60)
        text="YOU WIN"
        x=font.size(text)
        col=(255,255,255)
        self.winsurf.blit(font.render(text,1,col),(SW/2-x[0]/2,10))
        text="TIME ATTACK MODE ENABLED! PRESS Z TO GO BACK TO MENU"
        font=pygame.font.Font("copacetix.ttf",20)
        y=font.size(text)
        self.winsurf.blit(font.render(text,1,col),(SW/2-y[0]/2,10+x[1]))
        """

    def makewinscreen(self):
        self.winsurf=pygame.Surface((SW,SH/4),screen.get_flags(),screen)
        font=pygame.font.Font(df("copacetix.ttf"),60)
        text="YOU WIN"
        x=font.size(text)
        col=(255,255,255)
        self.winsurf.blit(font.render(text,1,col),(SW/2-x[0]/2,10))
        if not self.timeattackmode:
            text="TIME ATTACK MODE ENABLED! PRESS Z TO GO BACK TO MENU"
        else:
            text="IN "+str(g.totalturn)+" TURNS"+" AND PRESERVED THEIR UNIVERSE"*(g.totalturn<5000)
        font=pygame.font.Font(df("copacetix.ttf"),20)
        y=font.size(text)
        self.winsurf.blit(font.render(text,1,col),(SW/2-y[0]/2,10+x[1]))

    def levelup(self):
        self.coversurf.set_alpha(0)
        self.levelupy=SH
        self.levelupstate=0
        self.levelingup=True
        self.paused=LEVELUPPAUSE
        if self.level==7:
            self.wingame=True
            self.makewinscreen()

    def levelupanim(self,screen):
        if self.levelupstate==0:
            self.levelupy-=20
            alpha=self.coversurf.get_alpha()
            if alpha<255:
                self.coversurf.set_alpha(min(255,alpha+20))
            if self.levelupy<=3*SH/8:
                self.levelupstate+=1
        elif self.levelupstate>0 and self.levelupstate<LEVELUPDELAY:
            if debug: print self.coversurf.get_alpha()
            if not self.wingame: self.levelupstate+=1
        elif self.levelupstate==LEVELUPDELAY:
            self.loadlevel(self.level+1)
            self.levelupstate+=1
            self.paused=LEVELUPPAUSE
        else:
            self.levelupy-=20
            alpha=self.coversurf.get_alpha()
            if alpha>0:
                self.coversurf.set_alpha(max(0,alpha-20))
            if self.levelupy<=-SH/4:
                self.levelupstate+=1
                self.startlevel()
        screen.blit(self.coversurf,(0,0))
        if not self.wingame:
            screen.blit(self.levelupsurf,(0,self.levelupy))
        else:
            screen.blit(self.winsurf,(0,self.levelupy))

    def loadgameover(self):
        self.gameoversurf=pygame.Surface((SW,SH/4),screen.get_flags(),screen)
        self.gameoverfont=pygame.font.Font(df("copacetix.ttf"),60)
        text="GAME OVER"
        col=(255,255,255)
        self.gameoverlen=self.gameoverfont.size("GAME OVER ")
        self.gameoversurf.blit(self.gameoverfont.render(text,1,col),(0,10))
        #text="TO RESTART PRESS R"
        text="ART PRESS R TO REST"
        self.gameoverfont=pygame.font.Font(df("copacetix.ttf"),34)
        self.gameoversurf.blit(self.gameoverfont.render(text,1,col),(0,10+self.gameoverlen[1]))

    def gameover(self):
        self.shift=0
        self.paused=GAMEOVERPAUSE

    def drawgameover(self,screen):
        length=self.gameoverlen[0]
        self.shift=(self.shift+3)%length
        shift=-self.shift
        screen.blit(self.gameoversurf,(shift,SH*3/8))
        screen.blit(self.gameoversurf,(length+shift,SH*3/8))
        screen.blit(self.gameoversurf,(2*length+shift,SH*3/8))

    def loadpause(self):
        pausebox=pygame.Rect(40,20,SW-2*40,SH-2*20)
        self.psurf.fill([250,250,250])
        self.psurf.fill([255,255,255],pausebox)
        pygame.draw.rect(self.psurf,(0,0,0),pausebox,3)
        text="dt=0"
        col=(0,0,0)
        self.psurf.blit(DTIMG,(pausebox.left+pausebox.width/2-40,pausebox.top+4))
        s="\n\nLeft/right arrow\nZ\nR\nP\nS\nM\nESC,Q"
        rendertext(self.psurf,s,self.pausefont,pausebox)
        s="\n\nTurn\nContinue conversation\nRestart level\nPause/unpause\nToggle sound\nToggle music\nQuit to menu"
        rendertext(self.psurf,s,self.pausefont,pausebox.move(pausebox.width/2,0))

    def loadscript(self):
        #if debug: print "Loading script"
        f=open(df("script"))
        self.script=[]
        ev=[]
        s=""
        for line in f:
            if line=="=\n":
                self.script.append(ev)
                ev=[]
            elif len(line)==1:
                if len(s)>0:
                    ev.append(s)
                    s=""
            else:
                s+=line
        f.close()
        self.script.append(ev)
        for i in xrange(len(self.script)):
            if debug: print i,self.script[i]

    def loadlevel(self,level):
        self.turn=0
        self.level=level
        self.zsurf=pygame.Surface(boardsize,screen.get_flags(),screen)
        self.zsurf.set_colorkey((250,250,250))
        self.wcount=0
        self.tsurf=pygame.Surface((SW,SH),screen.get_flags(),screen)
        self.tsurf.set_colorkey((250,250,250))
        self.dialogvisible=False
        self.wingame=False
        self.parity=True

        self.mq=Quill((0,0),screen)
        self.stopsounds()

        if level==1: self.paused=1
        else: self.paused=0
        if level<100: self.scheme=allschemes[self.level]
        if level==1:
            self.zones=[]
            self.zones.append(Zone(80j,40,3))

            self.polyr=200
            self.polyoffset=0.5

            self.state=0
            self.wcountneed=1
        elif level==2:
            self.zones=[]
            self.zones.append(Zone(90-90j,20,0))
            self.zones.append(Zone(90+90j,20,0,1))
            self.zones.append(Zone(-90+90j,20,0,2))
            self.zones.append(Zone(-90-90j,20,0,3))
            self.zones.append(Zone(90+0j,20,0,4))
            self.zones.append(Zone(-90j,20,0,5))

            self.polyr=200
            self.polyoffset=0.5

            self.wcountneed=len(self.zones)
            self.state=10
        elif level==3:
            self.zones=[]
            self.zones.append(Zone(90-90j,20,0))
            self.zones.append(Zone(90+90j,20,0,1))
            self.zones.append(Zone(-90+90j,20,0,2))
            self.zones.append(Zone(-90-90j,20,0,3))

            self.polyr=200
            self.polyoffset=0.5

            self.wcountneed=len(self.zones)
            self.state=20
        elif level==4:
            self.zones=[]
            self.zones.append(Zone(130j,20,0))
            self.zones.append(Zone(-130j,20,0,1))
            self.zones.append(Zone(100+100j,20,0,2))
            self.zones.append(Zone(100-100j,20,0,3))
            self.zones.append(Zone(-100-100j,20,0,4))
            self.zones.append(Zone(-100+100j,20,0,5))
            self.zones.append(Zone(130-50j,20,0,6))

            self.polyr=200
            self.polyoffset=0.5

            self.wcountneed=len(self.zones)
            self.state=40
        elif level==5:
            self.polyr=200
            self.polyoffset=0.5

            self.zones=[]
            self.zones.append(Zone(polypolar(self.polyr-20,3+self.polyoffset,6),20,0))
            self.zones.append(Zone(polypolar(self.polyr-20,0+self.polyoffset,6),20,0,1))
            self.zones.append(Zone(polypolar(self.polyr-20,2+self.polyoffset,6),20,0,2))
            self.zones.append(Zone(polypolar(self.polyr-20,5+self.polyoffset,6),20,0,3))
            self.zones.append(Zone(polypolar(self.polyr-20,4+self.polyoffset,6),20,0,4))
            self.zones.append(Zone(polypolar(self.polyr-20,1+self.polyoffset,6),20,0,5))
            self.zones.append(Zone(0j,20,0,6))

            self.wcountneed=len(self.zones)
            self.state=50

        elif level==6:
            self.mq.xy+=50+50j

            self.polyr=200
            self.polyoffset=0.5

            self.zones=[]
            self.zones.append(Zone(polypolar(self.polyr-40,0.5+self.polyoffset,6),20,0))
            self.zones.append(Zone(polypolar(self.polyr-40,3.5+self.polyoffset,6),20,0,1))
            self.zones.append(Zone(polypolar(self.polyr-40,1.5+self.polyoffset,6),20,0,2))
            self.zones.append(Zone(polypolar(self.polyr-40,2.5+self.polyoffset,6),20,0,3))

            self.wcountneed=len(self.zones)
            self.state=60
            init=polypolar(self.polyr,2+self.polyoffset,6) #self.polyr*cexp(2*pi*(2+self.polyoffset)*1j/6)
            final=polypolar(self.polyr,5+self.polyoffset,6) #self.polyr*cexp(2*pi*(5+self.polyoffset)*1j/6)
            if debug: print "Manually added",init,final
            self.mq.trail.append(Segm(init,final,diffmode=False))
        elif level==7:
            self.polyr=200
            self.polyoffset=0.5

            self.zones=[]
            self.zones.append(Zone(polypolar(self.polyr-20,0+self.polyoffset,8),20,0))
            self.zones.append(Zone(polypolar(self.polyr-70,1+self.polyoffset,8),20,0,1))
            self.zones.append(Zone(polypolar(self.polyr-20,2+self.polyoffset,8),20,0,2))
            self.zones.append(Zone(polypolar(self.polyr-70,3+self.polyoffset,8),20,0,3))
            self.zones.append(Zone(polypolar(self.polyr-20,4+self.polyoffset,8),20,0,4))
            self.zones.append(Zone(polypolar(self.polyr-70,5+self.polyoffset,8),20,0,5))
            self.zones.append(Zone(polypolar(self.polyr-20,6+self.polyoffset,8),20,0,6))
            self.zones.append(Zone(polypolar(self.polyr-70,7+self.polyoffset,8),20,0,7))
            self.zones.append(Zone(polypolar(self.polyr-20,5+self.polyoffset,8),20,0,8))
            self.zones.append(Zone(polypolar(self.polyr-20,1+self.polyoffset,8),20,0,9))
            self.zones.append(Zone(polypolar(self.polyr-20,7+self.polyoffset,8),20,0,10))
            self.zones.append(Zone(polypolar(self.polyr-20,3+self.polyoffset,8),20,5,11))

            self.wcountneed=len(self.zones)+1
            self.state=70

        elif level==100:
            self.mq.lives=2
            self.zones=[]
            self.zones.append(Zone(-100j,40,100))
            self.zones.append(Zone(-100+0j,40,100))
            self.zones.append(Zone(100j,40,100))

            self.scheme="a+b+c+a+b+c+"

            self.polyn=6
            self.polyr=200
            self.polyoffset=0

            self.state=0
        #self.mq.lives=2

        self.polyn,self.bjump,self.bflip,arrows=schemetopoly(self.scheme)
        self.boundaries=[]
        for k in xrange(self.polyn):
            init=self.polyr*cexp(2*pi*(k+self.polyoffset)*1j/self.polyn)
            final=self.polyr*cexp(2*pi*(k+self.polyoffset+1)*1j/self.polyn)
            mink=min(k,self.bjump[k])
            if not self.bflip[k] and k==mink:
                init,final=final,init
            self.boundaries.append(Segm(init,final,diffmode=False,arrow=arrows[k]))

        self.bsurf=pygame.Surface(boardsize,screen.get_flags(),screen)
        self.bsurf.set_colorkey((250,250,250))
        self.bsurf.fill([250,250,250])

        for b in self.boundaries: b.draw(self.bsurf)
        self.updatez()

    def startlevel(self):
        self.playbgm()
        if self.level==2:
            self.triggerevent(11)
        elif self.level==3:
            self.triggerevent(21)
        elif self.level==4:
            self.triggerevent(40)
        elif self.level==5:
            self.triggerevent(50)
        elif self.level==6:
            self.triggerevent(60)
        elif self.level==7:
            self.triggerevent(70)

    def restartlevel(self):
        self.loadlevel(self.level)
        self.startlevel()

    def checkwin(self):
        if self.level in [1,2,3,4,5,6,7] and self.wcount==self.wcountneed:
            #self.loadlevel(self.level+1)
            #self.startlevel()
            self.levelup()
            return True

    def drawtextbox(self,texts=""):
        self.tsurf.fill([250,250,250])
        self.tsurf.fill([255,255,255],textbox)
        pygame.draw.rect(self.tsurf,(0,0,0),textbox,3)
        col=(0,0,0)
        l=0
        if texts.startswith("[Quill]"):
            font=self.quillfont
        else:
            font=self.dialogfont
        h=font.size(' ')[1]
        for s in texts.split("\n"):
            if "S^20" in s:
                offset=font.size(s[:s.index("S^20")])[0]
                #font.render(s[:ind],1,col)
                text=font.render(s.replace("S^20","      "),1,col)
                #text=font.render(" "*len(s),1,col)
                self.tsurf.blit(text,(textbox.left+4,textbox.top+4+h*l))
                self.tsurf.blit(S20IMG,(textbox.left+4+offset,textbox.top+4+h*l))
            elif "R^n" in s:
                offset=font.size(s[:s.index("R^n")])[0]
                #font.render(s[:ind],1,col)
                text=font.render(s.replace("R^n","     "),1,col)
                #text=font.render(" "*len(s),1,col)
                self.tsurf.blit(text,(textbox.left+4,textbox.top+4+h*l))
                self.tsurf.blit(RnIMG,(textbox.left+4+offset,textbox.top+4+h*l))
            else:
                text=font.render(s,1,col)
                self.tsurf.blit(text,(textbox.left+4,textbox.top+4+h*l))
            l+=1

    def updatez(self):
        #if debug: print "updatez",self.zones
        self.zsurf.fill([250,250,250])
        for z in self.zones:
            if z.active: z.draw(self.zsurf)

    def talkevent(self,ev,st,si,low,end):
        self.dialogvisible=True
        self.paused=3
        self.waiting=True

        self.state=st
        if ev==low: self.scriptstate=0
        else:
            self.scriptstate+=1
            if self.scriptstate==len(self.script[si]):
                self.state+=1
                return self.triggerevent(end)
        s=self.script[si][self.scriptstate]
        self.drawtextbox(s)

    def triggerevent(self,ev):
        if ev not in [24,28] and debug: print "Trigger",ev
        if ev in [1,2]:
            self.talkevent(ev,1,1,1,4)
        elif ev==3:
            self.dialogvisible=False
            self.paused=0
            self.waiting=False
        elif ev==4:
            self.triggerevent(3)
            self.paused=4
            self.state=3
            self.zones[-1].active=False
            self.zones.append(Zone(100j,40,4))
        elif ev==5:
            self.zones[-1].center+=4
            self.zones[-1].r-=0.2
            self.updatez()
            if self.zones[-1].center.real >= 200:
                self.paused=0
                self.state=4
                self.waiting=False
        elif ev in [6,7]:
            self.talkevent(ev,5,2,6,8)
        elif ev==8:
            self.triggerevent(3)
            self.zones[-1].active=False
            self.zones.append(Zone(-100j,20,0))
            self.paused=0
            self.dialogvisible=False
            self.waiting=False

            #self.zones.append(Zone(-100+0j,20,1))
            #self.zones.append(Zone(-150+0j,20,2,2))
            self.updatez()
        elif ev in [11,12]:
            self.talkevent(ev,11,3,11,3)
        elif ev in [21,22]:
            self.talkevent(ev,21,4,21,23)
        elif ev==23:
            self.paused=5
            #self.talkevent(ev,23,7,23,3)
            self.special=[Shape4d(1,xy=(-100,-100)),Shape4d(2,xy=(100,0)),Shape4d(3,xy=(500,0))]
            self.asurf.set_alpha(0)
            self.triggerevent(24)
            self.waiting=False
        elif ev==24:
            self.paused=5
            self.asurf.fill([255,255,255])
            self.special[0].x+=1
            self.special[0].y+=1
            self.special[1].x+=1
            for s in self.special:
                s.draw()
                s.rotate()
                self.asurf.blit(s.surf,(s.x,s.y))
            if self.special[0].x==0:
                self.triggerevent(25)
        elif ev==25:
            self.special[0].y-=100
            self.special[1].x-=30
            self.special[1].y-=100
            self.playsound("transport")
            self.talkevent(ev,25,5,25,26)
            self.paused=6
        elif ev==26:
            if self.scriptstate in [0]:
                self.special[0].x-=20
                self.special[0].y+=50
                self.special[1].x+=30
                self.special[1].y+=50
                self.playsound("transport")
            elif self.scriptstate in [2,7]:
                self.special[0].x-=60
                self.special[0].y-=150
                self.special[1].x+=130
                self.special[1].y+=50
                self.special[2].x-=110
                self.special[2].y-=150
                self.playsound("transport")
            elif self.scriptstate in [5,9]:
                self.special[0].x-=60
                self.special[0].y+=150
                self.special[1].x-=130
                self.special[1].y-=50
                self.special[2].x-=60
                self.special[2].y+=150
                self.playsound("transport")
            self.talkevent(ev,25,5,25,30)
            if self.state in [25,26]:
              if self.scriptstate==1:
                self.paused=7
                self.waiting=False
                self.state=29
              else:
                self.paused=6
            
        elif ev==28:
            self.paused=6
            self.asurf.fill([255,255,255])
            for s in self.special:
                s.draw()
                s.rotate()
                self.asurf.blit(s.surf,(s.x,s.y))
        elif ev==29:
            self.special[2].x-=3
            self.paused=7
            self.asurf.fill([255,255,255])
            for s in self.special:
                s.draw()
                s.rotate()
                self.asurf.blit(s.surf,(s.x,s.y))
            if self.special[2].x<=SW-200-100:
                self.state=25
                self.triggerevent(26)
        elif ev==30:
            self.state=30
            self.waiting=False
            self.paused=8
            self.asurf.fill([255,255,255])
            for s in self.special:
                s.draw()
                s.rotate()
                self.asurf.blit(s.surf,(s.x,s.y))
        elif ev==31:
            self.triggerevent(3)
            self.state=31
        elif ev in [32,33]:
            self.talkevent(ev,32,6,32,3)
        elif ev in [40,41]:
            self.talkevent(ev,40,7,40,3)
        elif ev in [50,51]:
            self.talkevent(ev,50,8,50,3)
        elif ev in [60,61]:
            self.talkevent(ev,60,9,60,3)
        elif ev in [70,71]:
            self.talkevent(ev,70,10,70,3)
        elif ev in [72,73]:
            self.talkevent(ev,72,11,72,3)

    def pausetrigger(self,val):
        if val==4: self.triggerevent(5) 
        else: self.triggerevent(val)
        
    def triggerz(self):
        if self.level==1 and self.state==1:
            self.triggerevent(2)
        elif self.level==1 and self.state==5:
            self.triggerevent(7)
        elif self.level==2 and self.state==11:
            self.triggerevent(12)
        elif self.level==3 and self.state==21:
            self.triggerevent(22)
        elif self.level==3 and self.state==25:
            self.triggerevent(self.state+1)
        elif self.level==3 and self.state==32:
            self.triggerevent(33)
        elif self.level==4 and self.state==40:
            self.triggerevent(41)
        elif self.level==5 and self.state==50:
            self.triggerevent(51)
        elif self.level==6 and self.state==60:
            self.triggerevent(61)
        elif self.level==7 and self.state==70:
            self.triggerevent(71)
        elif self.level==7 and self.state==72:
            self.triggerevent(73)

    def updatehud(self):
        self.hudsurf.fill([250,250,250])
        text = g.scorefont.render("w="+str(g.wcount),1,(0, 0, 0))
        g.hudsurf.blit(text,(0,0))
        s="z "*(g.level>JUMPLEVEL and g.turn>g.mq.nextjump)+"x "*g.mq.lives
        text = g.scorefont.render(s,1,(0, 0, 0))
        g.hudsurf.blit(text,(SW-g.scorefont.size(s)[0],0))
        if g.timeattackmode:
            text = g.scorefont.render("t="+str(g.totalturn),1,(0, 0, 0))
            g.hudsurf.blit(text,(SW/2-g.scorefont.size(" ")[0]*5,0))

    def loadsounds(self):
        self.sounds["wup"]=pygame.mixer.Sound(df("info.wav"))
        self.sounds["transport"]=pygame.mixer.Sound(df("teleport.ogg"))
        #pygame.mixer.Sound(df("/usr/share/games/vegastrike/sounds/Flux.wav")
        #self.sounds["transport"].set_volume(0.8)
        self.sounds["clear"]=pygame.mixer.Sound(df("slide.wav"))
        self.sounds["cross"]=pygame.mixer.Sound(df("shieldhit2.wav"))
        self.sounds["bgm"]=pygame.mixer.Sound(df("ocean.ogg"))

        self.sounds["menumove"]=pygame.mixer.Sound(df("info.wav"))
        self.sounds["menuselect"]=pygame.mixer.Sound(df("teleport.ogg"))

    def playbgm(self):
        if self.bgmon: self.sounds["bgm"].play(-1)

    def playsound(self,s):
        if self.soundon and s in self.sounds:
            self.sounds[s].play()

    def stopsounds(self):
        for s in self.sounds.values(): s.fadeout(1000)

    def togglesound(self):
        self.soundon = not self.soundon

    def togglebgm(self):
        self.bgmon = not self.bgmon
        if not self.bgmon: self.sounds["bgm"].stop()

    def soundstate(self):
        return "on"*self.soundon+"off"*(not self.soundon)

    def bgmstate(self):
        return "on"*self.bgmon+"off"*(not self.bgmon)

class Quill:
    def __init__(self,xy,origsurf):
        self.xy=xy[0]+xy[1]*1J
        self.trail=[]
        self.trailsurf=pygame.Surface(boardsize,origsurf.get_flags(),origsurf)
        self.trailsurf.fill([250,250,250])
        self.trailsurf.set_colorkey((250,250,250))

        self.lastdraw=len(self.trail)
        self.speed=BASESPEED
        self.angle=0
        self.image=pgt.scale(QUILLIMG,(27,80))
        self.invultime=0#g.turn

        self.z=0
        self.dz=0
        self.nextjump=0

        self.lives=0

    def cleartrails(self):
        self.trailsurf.fill([250,250,250])
        self.trailsurf.set_colorkey((250,250,250))
        self.trail=[]

    def rotate(self,clockwise):
        if self.z==0 and self.dz==0:
            self.angle+=clockwise*BASEROT

    def move(self,boundaries):
        newseg=Segm(self.xy,self.speed*cexp(self.angle*1j))
        #if any(newseg in x for x in self.trail[:-SAFEZONE]): if debug: print "Death"
        for b in boundaries:
            if newseg in b:
                if g.state==4:
                    if boundaries.index(b)==3:
                        g.triggerevent(6)
                elif g.state==31:
                    if boundaries.index(b) in [0,2]:
                        g.triggerevent(32)
                #if debug: print newseg.diff,newseg.diff.imag/newseg.diff.real
                #Need to rescan the new segment
                t1,t2,inpt=newseg.intpt(b)
                newb=boundaries[g.bjump[boundaries.index(b)]]
                if debug: print "jump", boundaries.index(b),g.bjump[boundaries.index(b)]
                outpt=newb.init+t2*newb.diff
                newseg1=Segm(self.xy,inpt,diffmode=False)

                #self.angle+=10
                #if debug: print newseg1.diff,newseg1.diff.imag/newseg1.diff.real
                if debug: print "t",t1,t2
                remdiff=(1-t1)*newseg.diff
                theta=anglebetween(remdiff,b.diff)
                if g.bflip[boundaries.index(b)]==1: theta=cc(theta)
                if debug: print theta,newb.diff,b.diff
                if debug: print "newdiff",theta.real*newb.diff*abs(remdiff)/abs(newb.diff),theta.imag*(newb.diff*(-1j))*abs(remdiff)/abs(newb.diff)
                newdiff=(theta.real*newb.diff*abs(remdiff)/abs(newb.diff)+theta.imag*(newb.diff*(-1j))*abs(remdiff)/abs(newb.diff))
                if debug: print outpt,newdiff
                self.angle=atan2(newdiff.imag,newdiff.real)
                #newdiff=newseg.diff
                #endpt=outpt+(1-t1)*abs(newseg.diff)*self.angle
                
                newseg=Segm(outpt,newdiff,diffmode=True)
                self.xy=outpt
                if self.z==0 and self.dz==0:
                    self.trail.append(newseg1)
                else:
                    self.updatez()
                g.parity=not g.parity
                g.updatez()
                if debug: print "parity",g.parity
                g.playsound("transport")
                break

        if self.z==0 and self.dz==0:
            if g.turn>self.invultime and any(newseg in x for x in self.trail[:-SAFEZONE]):
                if debug: print newseg,[x for x in self.trail[:-SAFEZONE] if newseg in x]
                self.invultime=g.turn+INVULTURNS
                g.playsound("cross")
                self.lives-=1
                if self.lives<0:
                    g.gameover()
                if debug: print "Death"
            self.trail.append(newseg)
        else:
            self.updatez()
        self.xy+=newseg.diff

    def updatez(self):
        self.z+=self.dz
        self.dz-=0.1
        if self.z<=0:
            self.dz=0
            self.z=0

    def draw(self,screen):
        #screen.blit(self.image,self.rect)
        
        #screen.blit(pgt.rotate(self.image,-self.angle*180/pi),(self.xy.real-camera.left+qoffset.real,self.xy.imag-camera.top+qoffset.imag))
        if not (g.turn <= self.invultime and g.turn%10>5):
            screen.blit(self.image,(self.xy.real-camera.left+qoffset.real,self.xy.imag-camera.top+qoffset.imag-self.z))

        #pygame.draw.circle(screen,(0,0,0),(self.xy.real-camera.left,self.xy.imag-camera.top),50,5)
        #if debug: print (self.xy.real-camera.left,self.xy.imag-camera.top)

    def drawtrail(self,screen):
        for i in xrange(self.lastdraw,len(self.trail)):
            self.trail[i].draw(self.trailsurf)
        self.lastdraw=len(self.trail)
        screen.blit(self.trailsurf,(-camera.left-g.origin.real,-camera.top-g.origin.imag))
        #screen.blit(self.trailsurf,(-camera.left,-camera.top))

    def jump(self):
        if debug: print "Jumping",self.z
        if self.z==0 and g.turn>self.nextjump:
            self.dz=2
            self.nextjump=g.turn+JUMPDELAY
            #g.playsound("jump")

class Zone:
    def __init__(self,center,radius,type,delay=0):
        if debug: print "zone",center,radius
        self.center=center
        self.r=radius
        self.type=type
        self.active=True
        self.delay=delay

    def draw(self,screen):
        if self.type==1 or self.type==101:
            if g.parity: col=(0,0,0)
            else: col=(127,127,127)
        elif self.type==2 or self.type==102:
            if not g.parity: col=(0,0,0)
            else: col=(127,127,127)
        else:
            col=(0,0,0)
        if self.delay>g.wcount:
            col=(127,127,127)
            self.drawtext(str(self.delay-g.wcount),screen,col)
        if self.type==3:
            self.drawtext("Help",screen,col)
        elif self.type==6:
            self.drawtext("Jump",screen,col)

        if self.type in [1,101]:
            for i in xrange(6):
                pygame.draw.arc(screen,col,(c2t(self.center-self.r-1j*self.r+g.origin),(2*self.r,2*self.r)),2*i*arcangle+(g.turn%180)*CIRCSPIN,(2*i+1)*arcangle+(g.turn%180)*CIRCSPIN,3)
        elif self.type in [2,102]:
            for i in xrange(6):
                pygame.draw.arc(screen,col,(c2t(self.center-self.r-1j*self.r+g.origin),(2*self.r,2*self.r)),2*i*arcangle-(g.turn%180)*CIRCSPIN,(2*i+1)*arcangle-(g.turn%180)*CIRCSPIN,3)
        else:
            pygame.draw.circle(screen,col,c2t(self.center+g.origin),self.r,3)
    def drawtext(self,s,screen,col):
        text=g.delayfont.render(s,1,col)
        ts=g.delayfont.size(s)
        screen.blit(text,c2t(self.center+g.origin-ts[0]/2-ts[1]/2*1j))

    def __contains__(self,other):
        #"in" means inside the zone here.
        if self.delay>g.wcount: return False
        if self.type==1 or self.type==101:
            return g.parity and abs(other-self.center)<self.r
        elif self.type==2 or self.type==102:
            return not g.parity and abs(other-self.center)<self.r
        elif self.type==6:
            return (abs(other-self.center)<self.r and g.mq.z>0)
        else:
            return abs(other-self.center)<self.r

    def hit(self):
        if debug: print "Hit"
        g.playsound("wup")
        if self.type in [0,1,2]:
            self.active=False
            g.wcount+=1
        if self.type==3:
            #self.active=False
            g.triggerevent(1)
        elif self.type==5:
            self.active=False
            g.wcount+=1
            g.zones.append(Zone(0j,50,6))
            g.triggerevent(72)
        elif self.type==6:
            self.active=False
            g.wcount+=1
        #Survival mode
        if self.type>=100:
            self.active=False
            g.wcount+=1
            g.zones.append(Zone(random.randint(0,150)*cexp(1j*random.randint(0,360)*pi/90),random.randint(10,20),203-self.type))
            if g.wcount**0.5%1<1e-7:
                g.mq.cleartrails()
                g.playsound("clear")
                if g.wcount%2==0: g.zones.append(Zone(random.randint(0,150)*cexp(1j*random.randint(0,360)*pi/90),random.randint(10,20),101+random.randint(1,2)))

#@profile
def main():
    #screen.scroll(-100,-100)
    c = pygame.time.Clock()
    lastfps = 0
    #mq=g.mq
    #zsurf=g.zsurf
    #zsurf=pygame.Surface(boardsize,screen.get_flags(),screen)
    #zsurf.set_colorkey((250,250,250))

    #if debug: print boundaries
    g.startlevel()
    while 1:
        if g.paused==0:
            g.turn+=1
            if g.timeattackmode: g.totalturn+=1
        c.tick(30)
        if pygame.time.get_ticks() - lastfps > 1000:
            if debug: print "FPS: ", c.get_fps()
            #if debug: print mq.trail
            if debug: print g.mq.angle,len(g.mq.trail),g.parity,g.paused,g.wcountneed
            lastfps = pygame.time.get_ticks()

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(0)
            elif event.type == pygame.KEYDOWN:
                if event.key in [pygame.K_ESCAPE,pygame.K_q]:
                    #sys.exit(0)
                    if g.level>g.maxlevel and g.level<100:
                        g.maxlevel=g.level
                        f=open("savedata","w")
                        print >>f,allschemes[g.level]
                        f.close()
                        print "Game saved"
                    return 0
                elif event.key == pygame.K_z:
                    if g.waiting:
                        g.triggerz()
                    elif g.level>JUMPLEVEL:
                        g.mq.jump()
                    if g.wingame and g.levelupstate>0:
                        g.level+=1
                        if g.level>g.maxlevel and g.level<100:
                            g.maxlevel=g.level
                            f=open("savedata","w")
                            print >>f,allschemes[g.level]
                            f.close()
                            if debug: print "Game saved"
                        return 1

                elif event.key == pygame.K_r:
                    g.restartlevel()
                    c = pygame.time.Clock()
                    lastfps = 0
                    #mq=g.mq
                    continue
                elif event.key == pygame.K_p:
                    if g.paused==0: g.paused=1
                    elif g.paused==1: g.paused=0
                elif event.key == pygame.K_s:
                    g.togglesound()
                elif event.key == pygame.K_m:
                    g.togglebgm()
                elif event.key == pygame.K_l:
                    pygame.image.save(screen,"screenshot.png")
        if g.paused==0:
            pressed = pygame.key.get_pressed()
            if pressed[pygame.K_RIGHT]:
                g.mq.rotate(1)
            if pressed[pygame.K_LEFT]:
                g.mq.rotate(-1)

            g.mq.move(g.boundaries)
            hashit=False
            if g.mq.z==0 and g.mq.dz==0:
              for z in g.zones:
                if z.active and g.mq.xy in z:
                    hashit=True
                    z.hit()
            elif g.level==7 and g.state==73:
              for z in g.zones:
                if z.active and z.type==6 and g.mq.xy in z:
                    hashit=True
                    z.hit()
                
            if hashit:
                g.updatez()
                if g.checkwin():
                    c = pygame.time.Clock()
                    lastfps = 0
                    #mq=g.mq
                    continue
            #High quality only
            g.updatez()
        elif g.paused==4:
            g.pausetrigger(4)
                        

        #camera.left+=1
        screen.fill([255, 255, 255])
        #pygame.draw.line(screen,(0,0,0),(100,100),(200,200))
        #if debug: print screen.get_palette()
        screen.blit(g.bsurf,(-camera.left-g.origin.real,-camera.top-g.origin.imag))
        screen.blit(g.zsurf,(-camera.left-g.origin.real,-camera.top-g.origin.imag))

        g.mq.drawtrail(screen)
        g.mq.draw(screen)
        g.updatehud()
        screen.blit(g.hudsurf,(0,0))

        if g.paused in [5,6,7]:
            alpha=g.asurf.get_alpha()
            if alpha<255: g.asurf.set_alpha(min(255,alpha+10))
            else: g.waiting=True
            if g.turn%1==0:
                if g.paused==5:
                    g.pausetrigger(24)
                elif g.paused==6:
                    g.pausetrigger(28)
                elif g.paused==7:
                    g.pausetrigger(29)
            screen.blit(g.asurf,(0,0))
        elif g.paused==8:
            alpha=g.asurf.get_alpha()
            if alpha>0:
                g.asurf.set_alpha(max(0,alpha-10))
                if g.turn%1==0:
                    g.pausetrigger(30)
                screen.blit(g.asurf,(0,0))
            else:
                g.triggerevent(31)

        if g.paused==1: screen.blit(g.psurf,(0,0))

        if g.dialogvisible:
            screen.blit(g.tsurf,(0,0))
            #screen.blit(g.tsurf,textbox)
        #screen.blit(g.tsurf,textbox)
        #if debug: print screen.get_colorkey()
        #screen.blit(test,(0,0))

        #screen.blit(g.levelupsurf,(0,0))
        #g.drawgameover(screen)
        if g.paused==LEVELUPPAUSE:
            g.levelupanim(screen)
        elif g.paused==GAMEOVERPAUSE:
            g.drawgameover(screen)

        pygame.display.update((0,0,SW,SH))
        #pygame.display.flip()

baset=pi/180
scx=scy=50
cx=cy=250

def proj(p):
    return (scx*p[0]+cx,scy*p[1]+cy)

def twodrot(p,theta):
    return [cos(theta)*p[0]-sin(theta)*p[1],sin(theta)*p[0]+cos(theta)*p[1]]

def rot(p,axis=(0,1),theta=baset):
    rotd=twodrot((p[axis[0]],p[axis[1]]),theta)
    #if debug: print p
    #if debug: print p[:axis[0]] ,(rotd[0],) ,p[axis(0)+1:axis(1)] ,(rotd[1],), p[axis(1)+1:]
    newp=p[:axis[0]]+[rotd[0]]+p[axis[0]+1:axis[1]]+[rotd[1]]+p[axis[1]+1:]
    return newp

class Shape4d:
    def __init__(self,type,dim=4,xy=(0,0)):
        if type==1:
            #hypercube
            points=[[2*((x >> i)%2)-1 for i in xrange(dim)] for x in xrange(2**dim)]
            lines=[]
            for i in xrange(len(points)):
                for j in xrange(i):
                    if sum(points[i][k]!=points[j][k] for k in xrange(dim))==1:
                        lines.append((i,j))
        elif type==2:
            #simplex
            dim+=1
            points=[[-1.0/dim]*dim for x in xrange(dim)]
            for i in xrange(dim):
                points[i][i]=1
            lines=[(x,y) for x in xrange(len(points)) for y in xrange(i)]
        elif type==3:
            #hypercube dual
            points=[[0 for i in xrange(dim)] for x in xrange(2*dim)]
            for i in xrange(dim):
                points[i][i]=1
            for i in xrange(dim):
                points[i+dim][i]=-1

            lines=[]
            for i in xrange(len(points)):
                for j in xrange(i):
                    if not i-j==dim:
                        lines.append((i,j))

        self.points=points
        self.lines=lines
        self.dim=dim
        self.surf=pygame.Surface((SW,SH),screen.get_flags(),screen)
        self.surf.set_colorkey([250,250,250,])
        self.col=([0,0,0])
        self.x,self.y=xy

    def rotate(self):
        points=self.points
        for i in xrange(len(points)):
            points[i]=rot(points[i],(0,1),baset)
            points[i]=rot(points[i],(1,2),2*baset)
            points[i]=rot(points[i],(2,3),2.5*baset)
            points[i]=rot(points[i],(0,3),2*baset)
            points[i]=rot(points[i],(0,2),0.5*baset)

    def draw(self):
        self.surf.fill([250,250,250])
        #self.surf.fill([0,0,0])
        for l in self.lines:
            pygame.draw.line(self.surf,self.col,proj(self.points[l[0]]),proj(self.points[l[1]]))

class Menu:
    def __init__(self,menuitems,cursor=0):
        self.menuitems=menuitems
        self.cursor=cursor
        self.font=g.menufont
        self.col=(0,0,0)
        self.fontheight=self.font.size(' ')[1]

        self.surf=pygame.Surface(boardsize,screen.get_flags(),screen)
        self.surf.set_colorkey((250,250,250))
        self.left,self.top=(40,140)
        self.curspr=pgt.flip(pgt.scale(QUILLIMG,(21,60)),True,False)#self.font.render(">",1,self.col)
        self.cursep=(30,10)#self.font.size('W')[0]

    def prev(self):
        self.cursor-=1
        if self.cursor<0: self.cursor=len(self.menuitems)-1
        g.playsound("menumove")

    def next(self):
        self.cursor+=1
        if self.cursor>=len(self.menuitems): self.cursor=0
        g.playsound("menumove")

    def draw(self):
        self.surf.fill([250,250,250])
        l=0
        h=self.fontheight
        for s in self.menuitems:
            if s=="Sound":
                s+=" "+g.soundstate()
            elif s=="Music":
                s+=" "+g.bgmstate()
            text=self.font.render(s,1,self.col)
            self.surf.blit(text,(self.left,self.top+h*l))
            l+=1
        self.surf.blit(self.curspr,(self.left-self.cursep[0],self.top+h*self.cursor-self.cursep[1]))
        self.surf.blit(g.title,(10,0))

menuelems=[]
menuelems.append(["Start game","Sound","Music","Quit"])
menuelems.append(["Story mode","Survival mode","Back"])
menuelems.append(["Level 1"])
def menu(mlev,cursor=0):
    c = pygame.time.Clock()
    lastfps = 0
    m=Menu(menuelems[mlev],cursor)
    m.draw()
    if g.fadeonce:
        m.surf.set_alpha(0)
        g.fadeonce=False
    while 1:
        c.tick(30)
        if pygame.time.get_ticks() - lastfps > 1000:
            if debug: print "FPS: ", c.get_fps()
            lastfps = pygame.time.get_ticks()

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(0)
            elif event.type == pygame.KEYDOWN:
                if event.key in [pygame.K_ESCAPE,pygame.K_q]:
                    sys.exit(0)
                elif event.key == pygame.K_z:
                    g.playsound("menuselect")
                    return m.cursor#m.menuitems[m.cursor]
                elif event.key == pygame.K_UP:
                    m.prev()
                    m.draw()
                elif event.key == pygame.K_DOWN:
                    m.next()
                    m.draw()
        alpha=m.surf.get_alpha()
        if alpha is not None and alpha<255: m.surf.set_alpha(min(alpha+10,255))
        screen.fill([255, 255, 255])
        screen.blit(m.surf,(0,0))
        pygame.display.flip()

def pretitle():
    c = pygame.time.Clock()
    lastfps = 0
    g.turn=0
    col=[255,255,255]
    quotesurf=pygame.Surface(boardsize,screen.get_flags(),screen)
    rendertext(quotesurf,g.script[0][0],g.quotefont,pygame.Rect(30,120,SW,SH),col)
    rendertext(quotesurf,g.script[0][1],g.quotefont,pygame.Rect(230,220,SW,SH),col)
    quotesurf.set_alpha(255)
    while 1:
        c.tick(30)
        if pygame.time.get_ticks() - lastfps > 1000:
            if debug: print "FPS: ", c.get_fps(),g.turn
            lastfps = pygame.time.get_ticks()

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(0)
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    sys.exit(0)
                elif event.key == pygame.K_z:
                    #fade
                    quotesurf.set_alpha(min(quotesurf.get_alpha,250))
        g.turn+=1
        alpha=quotesurf.get_alpha()
        if g.turn>=300 and alpha==255: quotesurf.set_alpha(250)
        if alpha==0: return
        if alpha<255: quotesurf.set_alpha(alpha-5)
        screen.fill([0, 0, 0])
        screen.blit(quotesurf,(0,0))
        pygame.display.flip()
        
#if __name__=="__main__":
if True:
    pygame.mixer.pre_init(44100,-16,2,3072)
    pygame.init()
    pygame.mixer.init()
    screen = pygame.display.set_mode((SW, SH),pygame.DOUBLEBUF)
    pgt=pygame.transform
    #QUILLIMG=pgt.scale(pgt.rotate(pygame.image.load("test.png"),-90),(27,80))
    QUILLIMG=pygame.image.load(df("quillbig2.tga"))
    #pygame.image.save(QUILLIMG,"quillbig.tga")
    S20IMG=pgt.scale(pygame.image.load(df("s20.png")),(30,22))
    RnIMG=pgt.scale(pygame.image.load(df("rn.png")),(30,24))
    DTIMG=pygame.image.load(df("dt.png"))
    
    g=Game()
    try:
        f=open("savedata")
        g.maxlevel=allschemes.index(f.readline().strip())
        if debug: print "Loaded save",g.maxlevel
        f.close()
    except:
        g.maxlevel=1
    
    #camera=pygame.Rect(0,0,SW,SH)
    camera=pygame.Rect(-SW/2,-SH/2,SW,SH)
    #screen=pygame.display.set_mode((640, 480), pygame.DOUBLEBUF)
    pretitle()
    m=0
    while 1:
        m=menu(0,m)
        if m==0:
            while 1:
                if g.maxlevel==8:
                    menuelems[1]=["Story mode","Survival mode","Time attack","Back"]
                else:
                    menuelems[1]=["Story mode","Survival mode","Back"]
                subm=menu(1,0)
                if subm==0:
                    if g.maxlevel==1:
                        g.timeattackmode=False
                        g.loadlevel(1)
                        ret=main()
                        break
                    else:
                        menuelems[2]=["Level "+str(x+1) for x in range(min(7,g.maxlevel))]+["Back"]
                        subsubm=menu(2,0)
                        if debug: print subsubm,g.maxlevel
                        if subsubm<min(7,g.maxlevel):
                            g.loadlevel(subsubm+1)
                            ret=main()
                            break
                elif subm==1:
                    g.timeattackmode=False
                    g.loadlevel(100)
                    ret=main()
                    break
                elif menuelems[1][subm]=="Back":
                    break
                elif subm==2:
                    g.timeattackmode=True
                    g.totalturn=0
                    g.loadlevel(7)
                    ret=main()
                    break
        elif m==1:
            g.togglesound()
        elif m==2:
            g.togglebgm()
        elif m==3:
            sys.exit(0)
